<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skrooge_search" version="11">
<MenuBar>
  <Menu name="edit"><text>Edit</text>
	  <Action name="edit_find" group="edit_group_find"/>
	  <Separator group="edit_group_find"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
	<Action name="edit_find" group="main_group_other"/>
</ToolBar>
</kpartgui>
