<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_file" version="11">
<MenuBar>
  <Menu name="file"><text>File</text>
	  <Action name="file_new" />
	  <Action name="file_open" />
	  <Action name="file_open_recent" />
	  <Separator/>
	  <Action name="file_save" />
	  <Action name="file_save_as" />
	  <Separator/>
	  <Action name="file_change_password" />
	  <Separator/>
	  <Merge/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
	<text>File</text>
	<Action name="file_new" group="main_group_file"/>
	<Action name="file_open" group="main_group_file" />
	<Separator group="main_group_file"/>
	<Action name="file_save" group="main_group_file"/>
	<Separator/>
	<Merge/>
</ToolBar>
</kpartgui>
