<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_statistic" version="1">
<MenuBar>
<Menu name="edit"><text>Edit</text></Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
