<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skrooge_bank" version="2">
<MenuBar>
  <Menu name="edit"><text>Edit</text>
	  <Action name="edit_reconciliate" group="edit_group_1"/>
	  <Separator group="edit_group_1"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
