<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="skg_selectall" version="2" >
<MenuBar>
  <Menu name="edit"><text>Edit</text>
	  <Action name="edit_select_all" group="edit_group_select"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
</ToolBar>
</kpartgui>
